from bloonix.heaven import H
from bloonix.heaven.plugin.crud import CRUD

class Controller(CRUD):
    def startup(self):
        self.db = H.model.db.location_group
        self.add_crud_routes("/config/location-groups")
        H.route(
            "/config/location-groups/:id/hosts/member",
            "location_group_host_member")
        H.route(
            "/config/location-groups/:id/hosts/no-member",
            "location_group_host_no_member")
        H.route(
            "/config/location-groups/:id/hosts/:host_id/add",
            "location_group_host_add")
        H.route(
            "/config/location-groups/:id/hosts/:host_id/remove",
            "location_group_host_remove")

    def auto(self, opts):
        if "id" in opts:
            location_group = H.model.db.location_group.get(opts["id"])

            if not location_group:
                return H.plugin.error.set("err-600")

            H.cache["location_group"] = location_group

        if "host_id" in opts:
            host = H.model.db.host.get(opts["host_id"])
            if not host:
                return H.plugin.error.set("err-600")

        return True

    def list(self, opts):
        ret = H.model.db.location_group.query(
            query=H.req.get_query_data(),
            count="id",
            table=["location_group", ["id", "name", "description"]],
            order=["asc", "location_group.name"]
        )

        if ret.success == False:
            return H.plugin.error.set("err-606")

        H.stash.data = ret.data
        H.stash.total = ret.total

    def location_group_host_member(self, opts):
        ret = H.model.db.host.query(
            query=H.req.get_query_data(),
            count="id",
            table=["host", ["id", "hostname", "ipaddr1", "project_class", "location_class"]],
            join=[
                "inner", [
                    "host_location_group",
                    "host.id",
                    "host_location_group.host_id"
                ]
            ],
            condition=[
                "where", [
                    "host_location_group.location_group_id",
                    opts["id"]
                ]
            ],
            order=["asc", "host.hostname"]
        )

        if ret.success == False:
            return H.plugin.error.set("err-606")

        H.stash.data = ret.data
        H.stash.total = ret.total

    def location_group_host_no_member(self, opts):
        ret = H.model.db.host.query(
            query=H.req.get_query_data(),
            count="id",
            table=["host", ["id", "hostname", "ipaddr1", "project_class", "location_class"]],
            condition=[
                "where", ["host.id", "not in", {
                    "table": "host_location_group",
                    "columns": "host_id",
                    "condition": [
                        "where", [
                            "host_location_group.location_group_id",
                            opts["id"]
                        ]
                    ]
                }]
            ],
            order=["asc", "host.hostname"]
        )

        if ret.success == False:
            return H.plugin.error.set("err-606")

        H.stash.data = ret.data
        H.stash.total = ret.total

    def location_group_host_add(self, opts):
        if not H.plugin.token.check():
            return

        H.model.db.host_location_group.create({
            "location_group_id": opts["id"],
            "host_id": opts["host_id"]
        });

    def location_group_host_remove(self, opts):
        if not H.plugin.token.check():
            return

        H.model.db.host_location_group.delete_by_col(
            "location_group_id", opts["id"],
            "host_id", opts["host_id"]
        )
