from bloonix.heaven import H
from bloonix.heaven.plugin.crud import CRUD

class Controller(CRUD):
    def startup(self):
        self.db = H.model.db.regauth
        self.add_crud_routes("/config/registration/authkeys")
        base = "/config/registration/authkeys/:id"

        H.route("%s/contactgroups/member" % base, "contactgroups_member")
        H.route("%s/contactgroups/no-member" % base, "contactgroups_no_member")
        H.route("%s/contactgroups/:contactgroup_id/add" % base, "contactgroups_add")
        H.route("%s/contactgroups/:contactgroup_id/remove" % base, "contactgroups_remove")

        H.route("%s/groups/member" % base, "groups_member")
        H.route("%s/groups/no-member" % base, "groups_no_member")
        H.route("%s/groups/:group_id/add" % base, "groups_add")
        H.route("%s/groups/:group_id/remove" % base, "groups_remove")

        H.route("%s/location-groups/member" % base, "location_groups_member")
        H.route("%s/location-groups/no-member" % base, "location_groups_no_member")
        H.route("%s/location-groups/:location_group_id/add" % base, "location_groups_add")
        H.route("%s/location-groups/:location_group_id/remove" % base, "location_groups_remove")

        H.route("%s/templates/member" % base, "templates_member")
        H.route("%s/templates/no-member" % base, "template_no_member")
        H.route("%s/templates/:template_id/add" % base, "templates_add")
        H.route("%s/templates/:template_id/remove" % base, "template_remove")

    def auto(self, opts):
        if "contactgroup_id" in opts:
            if not H.model.db.contactgroup.get(opts["contactgroup_id"]):
                return H.plugin.error.set("err-600")

        if "group_id" in opts:
            if not H.model.db.group.get(opts["group_id"]):
                return H.plugin.error.set("err-600")

        if "location_group_id" in opts:
            if not H.model.db.location_group.get(opts["location_group_id"]):
                return H.plugin.error.set("err-600")

        if "template_id" in opts:
            if not H.model.db.template.get(opts["template_id"]):
                return H.plugin.error.set("err-600")

        return super().auto(opts)

    def list(self, opts):
        H.stash.data = self.db.fetchall(order=["asc", "name"])

    def contactgroups_member(self, opts):
        ret = H.model.db.contactgroup.query(
            query=H.req.get_query_data(),
            count="id",
            table=["contactgroup", ["id", "name"]],
            join=[
                "inner", [
                    "regauth_contactgroup",
                    "contactgroup.id",
                    "regauth_contactgroup.contactgroup_id"
                ]
            ],
            condition=["where", ["regauth_contactgroup.regauth_id", opts["id"]]],
            order=["asc", "contactgroup.name"]
        )

        if ret.success == False:
            return H.plugin.error.set("err-606")

        H.stash.data = ret.data
        H.stash.total = ret.total

    def contactgroups_no_member(self, opts):
        ret = H.model.db.contactgroup.query(
            query=H.req.get_query_data(),
            count="id",
            table=["contactgroup", ["id", "name"]],
            condition=[
                "where", ["contactgroup.id", "not in", {
                    "table": "regauth_contactgroup",
                    "columns": "contactgroup_id",
                    "condition": [
                        "where", ["regauth_contactgroup.regauth_id", opts["id"]]
                    ]
                }]
            ],
            order=["asc", "contactgroup.name"]
        )

        if ret.success == False:
            return H.plugin.error.set("err-606")

        H.stash.data = ret.data
        H.stash.total = ret.total

    def contactgroups_add(self, opts):
        if not H.plugin.token.check():
            return

        H.model.db.regauth_contactgroup.create({
            "contactgroup_id": opts["contactgroup_id"],
            "regauth_id": opts["id"]
        })

    def contactgroups_remove(self, opts):
        if not H.plugin.token.check():
            return

        H.model.db.regauth_contactgroup.delete_by_col(
            "contactgroup_id", opts["contactgroup_id"],
            "regauth_id", opts["id"]
        )

    def groups_member(self, opts):
        ret = H.model.db.group.query(
            query=H.req.get_query_data(),
            count="id",
            table=["group", ["id", "name"]],
            join=[
                "inner", [
                    "regauth_group",
                    "group.id",
                    "regauth_group.group_id"
                ]
            ],
            condition=["where", ["regauth_group.regauth_id", opts["id"]]],
            order=["asc", "group.name"]
        )

        if ret.success == False:
            return H.plugin.error.set("err-606")

        H.stash.data = ret.data
        H.stash.total = ret.total

    def groups_no_member(self, opts):
        ret = H.model.db.group.query(
            query=H.req.get_query_data(),
            count="id",
            table=["group", ["id", "name"]],
            condition=[
                "where", ["group.id", "not in", {
                    "table": "regauth_group",
                    "columns": "group_id",
                    "condition": [
                        "where", ["regauth_group.regauth_id", opts["id"]]
                    ]
                }]
            ],
            order=["asc", "group.name"]
        )

        if ret.success == False:
            return H.plugin.error.set("err-606")

        H.stash.data = ret.data
        H.stash.total = ret.total

    def groups_add(self, opts):
        if not H.plugin.token.check():
            return

        H.model.db.regauth_group.create({
            "group_id": opts["group_id"],
            "regauth_id": opts["id"]
        })

    def groups_remove(self, opts):
        if not H.plugin.token.check():
            return

        H.model.db.regauth_group.delete_by_col(
            "group_id", opts["group_id"],
            "regauth_id", opts["id"]
        )

    def location_groups_member(self, opts):
        ret = H.model.db.location_group.query(
            query=H.req.get_query_data(),
            count="id",
            table=["location_group", ["id", "name"]],
            join=[
                "inner", [
                    "regauth_location_group",
                    "location_group.id",
                    "regauth_location_group.location_group_id"
                ]
            ],
            condition=["where", ["regauth_location_group.regauth_id", opts["id"]]],
            order=["asc", "location_group.name"]
        )

        if ret.success == False:
            return H.plugin.error.set("err-606")

        H.stash.data = ret.data
        H.stash.total = ret.total

    def location_groups_no_member(self, opts):
        ret = H.model.db.location_group.query(
            query=H.req.get_query_data(),
            count="id",
            table=["location_group", ["id", "name"]],
            condition=[
                "where", ["location_group.id", "not in", {
                    "table": "regauth_location_group",
                    "columns": "location_group_id",
                    "condition": [
                        "where", ["regauth_location_group.regauth_id", opts["id"]]
                    ]
                }]
            ],
            order=["asc", "location_group.name"]
        )

        if ret.success == False:
            return H.plugin.error.set("err-606")

        H.stash.data = ret.data
        H.stash.total = ret.total

    def location_groups_add(self, opts):
        if not H.plugin.token.check():
            return

        H.model.db.regauth_location_group.create({
            "location_group_id": opts["location_group_id"],
            "regauth_id": opts["id"]
        })

    def location_groups_remove(self, opts):
        if not H.plugin.token.check():
            return

        H.model.db.regauth_location_group.delete_by_col(
            "location_group_id", opts["location_group_id"],
            "regauth_id", opts["id"]
        )

    def templates_member(self, opts):
        ret = H.model.db.template.query(
            query=H.req.get_query_data(),
            count="id",
            table=["template", ["id", "name"]],
            join=[
                "inner", [
                    "regauth_template",
                    "template.id",
                    "regauth_template.template_id"
                ]
            ],
            condition=["where", ["regauth_template.regauth_id", opts["id"]]],
            order=["asc", "template.name"]
        )

        if ret.success == False:
            return H.plugin.error.set("err-606")

        H.stash.data = ret.data
        H.stash.total = ret.total

    def template_no_member(self, opts):
        ret = H.model.db.template.query(
            query=H.req.get_query_data(),
            count="id",
            table=["template", ["id", "name"]],
            condition=[
                "where", ["template.id", "not in", {
                    "table": "regauth_template",
                    "columns": "template_id",
                    "condition": [
                        "where", ["regauth_template.regauth_id", opts["id"]]
                    ]
                }]
            ],
            order=["asc", "template.name"]
        )

        if ret.success == False:
            return H.plugin.error.set("err-606")

        H.stash.data = ret.data
        H.stash.total = ret.total

    def templates_add(self, opts):
        if not H.plugin.token.check():
            return

        H.model.db.regauth_template.create({
            "template_id": opts["template_id"],
            "regauth_id": opts["id"]
        })

    def template_remove(self, opts):
        if not H.plugin.token.check():
            return

        H.model.db.regauth_template.delete_by_col(
            "template_id", opts["template_id"],
            "regauth_id", opts["id"]
        )
