from bloonix.heaven import H
from bloonix.exception import BloonixDuplicationError

class Controller:
    def startup(self):
        H.route("/config/registration/hosts", "list")
        H.route("/config/registration/hosts/count", "count")
        H.route("/config/registration/hosts/activate/options", "activate_options")
        H.route("/config/registration/hosts/:id/activate", "activate")
        H.route("/config/registration/hosts/:id/update", "update")
        H.route("/config/registration/hosts/:id/options", "options")
        H.route("/config/registration/hosts/:id/delete", "delete")

    def auto(self, opts):
        if "id" in opts:
            H.cache["regauth_host"] = H.model.db.regauth_host.find("host_id", opts["id"])
            if not H.cache["regauth_host"]:
                return H.plugin.error.set("err-600")
        return True

    def count(self, opts):
        H.stash.data = H.model.db.regauth_host.count(count="host_id")

    def list(self, opts):
        ret = H.model.db.regauth_host.query(
            query=H.req.get_query_data(),
            count="host_id",
            parse_int=["regauth_id", "regauth_id"],
            table=[
                "regauth_host", [
                    "regauth_id",
                    "host_id",
                    "hostname",
                    "ipaddr1",
                    "ipaddr2",
                    "ipaddr3",
                    "ipaddr4",
                    "sysinfo",
                    "country_code",
                    "location_class",
                    "project_class",
                    "tags",
                    "variables",
                ]
            ],
            order=["asc", "regauth_host.hostname"]
        )

        if ret.success == False:
            return H.plugin.error.set("err-606")

        templates = H.model.db.template.fetchall(columns=["id", "name", "tags"])
        by_tag = {}
        for template in templates:
            for tag in template["tags"]:
                if tag not in by_tag:
                    by_tag[tag] = []
                by_tag[tag].append(template)

        for row in ret.data:
            row["groups"] = H.model.db.regauth_group.by_regauth_id(row["regauth_id"])
            row["templates"] = H.model.db.regauth_template.by_regauth_id(row["regauth_id"])
            row["location_groups"] = H.model.db.regauth_location_group.by_regauth_id(row["regauth_id"])
            row["contactgroups"] = H.model.db.regauth_contactgroup.by_regauth_id(row["regauth_id"])
            row["templates_by_tags"] = tags = []
            for tag in row["tags"]:
                if tag in by_tag:
                    for template in by_tag[tag]:
                        tags.append({"id": template["id"], "name": template["name"], "tag": tag})

        H.stash.data = ret.data
        H.stash.total = ret.total

    def activate_options(self, opts):
        H.model.db.regauth_host.init_v_add_hosts()
        H.stash.data = H.model.db.regauth_host.v_add_hosts.get_options()

    def activate(self, opts):
        if not H.plugin.token.check():
            return

        host = H.cache["regauth_host"]
        H.model.db.regauth_host.init_v_add_hosts()
        res = H.model.db.regauth_host.v_add_hosts.validate(ignore_missing=True)
        if res.has_failed:
            return H.plugin.error.set("err-610", failed=res.failed)

        tags = host["tags"]
        del host["tags"]

        regauth_id = host["regauth_id"]
        del host["regauth_id"]

        password = host["password"]
        del host["password"]

        host["id"] = host["host_id"]
        del host["host_id"]

        groups = H.model.db.regauth_group.by_regauth_id(regauth_id)
        templates = H.model.db.regauth_template.by_regauth_id(regauth_id)
        location_groups = H.model.db.regauth_location_group.by_regauth_id(regauth_id)
        contactgroups = H.model.db.regauth_contactgroup.by_regauth_id(regauth_id)

        for template in H.model.db.template.fetchall(columns=["id", "tags"]):
            for tag in tags:
                if tag in template["tags"]:
                    templates.append({"id": template["id"]})

        try:
            H.model.db.host.create(host)
            H.model.db.host_secret.create({"host_id": host["id"], "password": password})
        except BloonixDuplicationError:
            return H.plugin.error.set("err-609", add_message=host["hostname"])

        seen = []
        if "group_id" in res.data:
            for i in res.data["group_id"]:
                groups.append({"id": i})
        for group in groups:
            if group["id"] in seen:
                continue
            seen.append(group["id"])
            try:
                H.model.db.host_group.create({
                    "group_id": group["id"],
                    "host_id": host["id"],
                })
            except Exception: pass

        seen = []
        if "contactgroup_id" in res.data:
            for i in res.data["contactgroup_id"]:
                contactgroups.append({"id": i})
        for contactgroup in contactgroups:
            if contactgroup["id"] in seen:
                continue
            seen.append(contactgroup["id"])
            try:
                H.model.db.host_contactgroup.create({
                    "contactgroup_id": contactgroup["id"],
                    "host_id": host["id"],
                })
            except Exception: pass

        seen = []
        if "location_group_id" in res.data:
            for i in res.data["location_group_id"]:
                location_groups.append({"id": i})
        for location_group in location_groups:
            if location_group["id"] in seen:
                continue
            seen.append(location_group["id"])
            try:
                H.model.db.host_location_group.create({
                    "location_group_id": location_group["id"],
                    "host_id": host["id"],
                })
            except Exception: pass

        seen = []
        if "template_id" in res.data:
            for i in res.data["template_id"]:
                templates.append({"id": i})
        for template in templates:
            if template["id"] in seen:
                continue
            seen.append(template["id"])
            try:
                H.model.db.host_template.add_host(
                    template["id"], host["id"]
                )
            except Exception: pass

        H.model.db.regauth_host.delete_by_col("host_id", host["id"])

    def options(self, opts):
        H.stash.data = {
            "options": H.model.db.regauth_host.v_update.get_options(),
            "values": H.cache["regauth_host"],
        }

    def update(self, opts):
        if not H.plugin.token.check():
            return
        res = H.model.db.regauth_host.v_update.validate(ignore_missing=True)
        if res.has_failed:
            return H.plugin.error.set("err-610", failed=res.failed)
        H.model.db.regauth_host.update_by_col("host_id", opts["id"], res.data)

    def delete(self, opts):
        if not H.plugin.token.check():
            return
        H.model.db.regauth_host.delete_by_col("host_id", opts["id"])
