import time
from bloonix.dbi.crud import CRUDV
from bloonix.heaven.validator import Validator

class Dependency(CRUDV):
    def by_host_id(self, host_id):
        stmt, bind = self.sql.select(
            table=[
                "dependency", "*",
                "service_parameter", "service_name"
            ],
            join=[
                "inner", [
                    "service",
                    "dependency.service_id",
                    "service.id"
                ],
                "inner", [
                    "service_parameter",
                    "service.service_parameter_id",
                    "service_parameter.id"
                ]
            ],
            condition=[
                "where", ["dependency.host_id", host_id]
            ],
            order=["asc", "id"]
        )

        rows = self.dbi.fetchall(stmt, bind)
        hosts = {}
        services = {}

        for row in rows:
            if row["on_host_id"] not in hosts:
                host = self.schema.host.get(
                    row["on_host_id"], columns=["id", "hostname"])
                hosts[host["id"]] = host["hostname"]

            if row["on_service_id"] not in services:
                service = self.schema.service.get(
                    row["on_service_id"], columns=["id", "service_parameter_id"])
                service_parameter = self.schema.service_parameter.get(
                    service["service_parameter_id"], columns=["service_name"])
                services[service["id"]] = service_parameter["service_name"]

            row["on_hostname"] = hosts[row["on_host_id"]]
            row["on_service_name"] = services[row["on_service_id"]]

        return rows

    def get_service_ids(self, host_id):
        stmt, bind = self.sql.select(
            table=[
                "dependency", ["host_id", "service_id", "on_service_id"],
                "service_parameter", "service_name",
                "host", "hostname",
            ],
            join=[
                "inner", [
                    "service",
                    "dependency.service_id",
                    "service.id"
                ],
                "inner", [
                    "service_parameter",
                    "service.service_parameter_id",
                    "service_parameter.id"
                ],
                "inner", [
                    "host",
                    "service.host_id",
                    "host.id",
                ],
            ],
            condition=[
                "where", ["dependency.host_id", host_id]
            ],
        )
        return self.dbi.fetchall(stmt, bind)

    def get_on_service_id(self, service_id):
        stmt, bind = self.sql.select(
            table=[
                "dependency", "on_service_id"
            ],
            condition=[
                "where", ["dependency.service_id", service_id]
            ]
        )
        return self.dbi.fetchall(stmt, bind)

    def by_id_and_host_id(self, dependency_id, host_id):
        stmt, bind = self.sql.select(
            table="dependency",
            columns="*",
            condition=[
                "where", ["id", dependency_id],
                "and", ["host_id", host_id]
            ])
        return self.dbi.fetchone(stmt, bind)
