import time
import bloonix.rx as rx
from bloonix.heaven import H
from bloonix.dbi.crud import CRUDV
from bloonix.heaven.validator import Validator
from bloonix import timeperiod
from bloonix import timezone

class HostDowntime(CRUDV):
    def init_validator(self, _type, host_id=None):
        v = Validator()
        self._set_validate_base(v)

        if _type == "fixtime":
            self._set_validate_fixtime(v)
        elif _type == "timeslice":
            self._set_validate_timeslice(v)
        elif _type == "preset":
            self._set_validate_preset(v)

        if host_id != None:
            self._set_validate_service_ids(v, host_id)

        return v

    def _set_validate_base(self, v):
        v.set({
            "description": {
                "type": str,
                "maxlength": 300
            },
            "timezone": {
                "type": str,
                "list_of_dicts_key": "value",
                "list_of_dicts_options": timezone.as_form(),
                "default": H.user["timezone"]
            }
        })

    def _set_validate_fixtime(self, v):
        v.set({
            "begin": {
                "type": str,
                "regex": rx.DATEHOURMIN,
                "isa": "unixtime"
            },
            "end": {
                "type": str,
                "regex": rx.DATEHOURMIN,
                "isa": "unixtime"
            }
        })

    def _set_validate_timeslice(self, v):
        v.set({
            "timeslice": {
                "type": list,
                "function": timeperiod.validate
            }
        })

    def _set_validate_preset(self, v):
        v.set({
            "preset": {
                "type": int,
                "options": [
                    3600, 7200, 14400, 28800, 57600,
                    86400, 172800, 604800, 1209600
                ]
            }
        })

    def _set_validate_service_ids(self, v, host_id):
        v.set({
            "service_ids": {
                "type": list,
                "list_of_dicts_key": "id",
                "list_of_dicts_multioptions": self.schema.service.by_host_id(host_id)
            }
        })

    def by_host_id(self, host_id):
        stmt, bind = self.sql.select(
            table=["host_downtime", "*"],
            condition=["where", ["host_id", host_id]],
            order=["asc", "id"])
        rows = self.dbi.fetchall(stmt, bind)
        unixtime = time.time()
        for row in rows:
            row["active"] = 0
            if len(row["timeslice"]):
                for timeslice in row["timeslice"]:
                    if timeperiod.check(timeslice):
                        row["active"] = 1
                        break
            elif row["begin"] < unixtime and row["end"] > unixtime:
                row["active"] = 1
        return rows

    def by_id_and_host_id(self, host_downtime_id, host_id):
        stmt, bind = self.sql.select(
            table="host_downtime",
            columns="*",
            condition=[
                "where", ["id", host_downtime_id],
                "and", ["host_id", host_id]
            ])
        return self.dbi.fetchone(stmt, bind)
