from bloonix.dbi.crud import CRUDV
from bloonix.heaven.validator import Validator
from bloonix import utils
from bloonix import netaddr

class LocationGroup(CRUDV):
    def __init__(self):
        self.validator = Validator(
            post_validation=self.__post_validation)
        self.validator.set({
            "name": {
                "type": str,
                "minlength": 1,
                "maxlength": 100,
            },
            "description": {
                "type": str,
                "minlength": 0,
                "maxlength": 100,
                "optional": True,
            },
            "password": {
                "type": str,
                "minlength": 30,
                "maxlength": 512,
            },
            "allow_from": {
                "type": list,
                "allow_empty": True,
                "maxsize": 10,
                "maxlength": 500,
                "default": [],
            },
        })

    def __post_validation(self, data):
        if "allow_from" in data:
            for ip in data["allow_from"]:
                if not netaddr.validate_ip(ip):
                    return {"allow_from": "err-711"}
