from bloonix.dbi.crud import CRUD
from bloonix.heaven import H
from bloonix.heaven.validator import Validator
from bloonix.json import json
import re

class SysOptions(CRUD):
    def __init__(self):
        def check_regex(value):
            try:
                re.compile("[%s]" % value)
                return True
            except Exception as err:
                return False

        self.validator = Validator()
        self.validator.set({
            "has_maintenance": {
                "type": int,
                "options": [0,1]
            },
            "event_history": {
                "type": int,
                "minvalue": 1,
                "maxvalue": 1200
            },
            "stat_history": {
                "type": int,
                "minvalue": 1,
                "maxvalue": 1200
            },
            "stat_hourly_history": {
                "type": int,
                "minvalue": 1,
                "maxvalue": 1200
            },
            "result_history": {
                "type": int,
                "minvalue": 1,
                "maxvalue": 1200
            },
            "notification_history": {
                "type": int,
                "minvalue": 1,
                "maxvalue": 3650
            },
            "user_tracking_history": {
                "type": int,
                "minvalue": 0,
                "maxvalue": 3650
            },
            "server_names": {
                "type": str,
            },
            "force_email_usernames": {
                "type": int,
                "options": [0,1]
            },
            "min_password_length": {
                "type": int,
                "minvalue": 1,
                "maxvalue": 255
            },
            "max_password_length": {
                "type": int,
                "minvalue": 1,
                "maxvalue": 255
            },
            "password_policy": {
                "allow_empty": True,
                "multioptions": [
                    "letter",
                    "lowercase",
                    "uppercase",
                    "number",
                    "special"
                ]
            },
            "password_policy_letter": {
                "type": str,
                "function": check_regex
            },
            "password_policy_lowercase": {
                "type": str,
                "function": check_regex
            },
            "password_policy_uppercase": {
                "type": str,
                "function": check_regex
            },
            "password_policy_number": {
                "type": str,
                "function": check_regex
            },
            "password_policy_special": {
                "type": str,
                "function": check_regex
            }
        })

    def _parse_options(self, rows):
        data = {}
        for row in rows:
            key = row["key"]
            value = row["value"]
            data_type = row["data_type"]
            if data_type == "int":
                data[key] = int(value)
            elif data_type == "float":
                data[key] = float(value)
            elif data_type == "json":
                data[key] = json.loads(value)
            elif data_type == "string":
                data[key] = value
        return data

    def get_all_keys(self):
        return self._parse_options(self.fetchall());

    def get_keys(self, *params):
        kwargs = {}
        if len(params):
            kwargs["condition"] = ["where", ["key", "in", list(params)]]
        return self._parse_options(self.fetchall(**kwargs))

    def get_keys_by_match(self, *match):
        cond = []
        for m in match:
            H.log.info(m)
            if len(cond):
                cond.append("or")
            else:
                cond.append("where")
            cond.append(["key", "like", m])
        return self._parse_options(self.fetchall(condition=cond))

    def update_keys(self, keys):
        return

    def has_maintenance(self):
        row = self.fetchone(condition=["where", ["key", "has_maintenance"]])
        if int(row["value"]) == 0:
            return 0
        return 1
