from bloonix.heaven import heaven
import json
import time
import decimal
import datetime

class Audit:
    def __init__(self):
        self._enabled = False
        self._history = 30

    def login(self, **kwargs):
        return self.log(action="login", **kwargs)

    def logout(self, **kwargs):
        return self.log(action="logout", **kwargs)

    def read(self, **kwargs):
        return self.log(action="read", **kwargs)

    def create(self, **kwargs):
        return self.log(action="create", **kwargs)

    def update(self, **kwargs):
        return self.log(action="update", **kwargs)

    def delete(self, **kwargs):
        return self.log(action="delete", **kwargs)

    def log(self, action, target=None, old_data=None, new_data=None, message=None):
        history = time.time() - self._history * 86400
        heaven.model.db.user_tracking.delete([
            "where", ["time", "<=", history]
        ])

        if self._enabled == False:
            return

        _old_data = None
        _new_data = None

        if isinstance(old_data, dict) and isinstance(new_data, dict):
            _old_data = {}
            _new_data = {}
            for key in new_data:
                if key in old_data and old_data[key] != new_data[key]:
                    heaven.log.notice(key, old_data[key], "!=", new_data[key])
                    _old_data[key] = old_data[key]
                    _new_data[key] = new_data[key]

        if _old_data != None:
            _old_data = json.dumps(_old_data, default=self.convert)
        elif old_data and isinstance(old_data, dict):
            _old_data = json.dumps(old_data, default=self.convert)

        if _new_data != None:
            _new_data = json.dumps(_new_data, default=self.convert)
        elif new_data and isinstance(new_data, dict):
            _new_data = json.dumps(new_data, default=self.convert)

        heaven.model.db.user_tracking.create({
            "user_id": heaven.user["id"],
            "username": heaven.user["username"],
            "time": int(time.time()),
            "action": action,
            "target": target,
            "message": message,
            "old_data": _old_data,
            "new_data": _new_data
        })

    def convert(self, o):
        if isinstance(o, decimal.Decimal):
            return float(o)
        if isinstance(o, datetime.datetime):
            return o.isoformat()
